#include "MeshLoader.h"

//  -  ,   
std::vector<Node>& MeshLoader::getNodes()
{
	return m_Nodes;
}

std::vector<FiniteElement>& MeshLoader::getFiniteElements()
{
	return m_FiniteElements;
}

std::vector<BoundaryFiniteElement>& MeshLoader::getBoundaryFiniteElements()
{
	return m_BoundaryFiniteElements;
}

//,     ID    
std::vector<FiniteElement> MeshLoader::findFiniteElementsByNodesID(int a_id1, int a_id2, int a_id3)
{
	std::vector<FiniteElement> result;
	std::vector<FiniteElement>::iterator currentIter = m_FiniteElements.begin();
	std::vector<FiniteElement>::iterator endIter = m_FiniteElements.end();

	while (currentIter != endIter)
	{
		currentIter = std::find_if(currentIter, endIter, [a_id1, a_id2, a_id3](const FiniteElement& elem) {
				
			return std::find(elem.m_nodes_id.begin(), elem.m_nodes_id.end(), a_id1) != elem.m_nodes_id.end()
				&& std::find(elem.m_nodes_id.begin(), elem.m_nodes_id.end(), a_id2) != elem.m_nodes_id.end()
				&& std::find(elem.m_nodes_id.begin(), elem.m_nodes_id.end(), a_id3) != elem.m_nodes_id.end();

			});

		if (currentIter != endIter)
		{
			result.push_back(*currentIter);
			currentIter++;
		}
	}

	return result;
}

//,     ,    ID  
std::vector<FiniteElement> MeshLoader::findFiniteElementsByNodesID(int a_id1, int a_id2)
{
	std::vector<FiniteElement> result;
	std::vector<FiniteElement>::iterator currentIter = m_FiniteElements.begin();
	std::vector<FiniteElement>::iterator endIter = m_FiniteElements.end();

	while (currentIter != endIter)
	{
		currentIter = std::find_if(currentIter, endIter, [a_id1, a_id2](const FiniteElement& elem) {

			return std::find(elem.m_nodes_id.begin(), elem.m_nodes_id.end(), a_id1) != elem.m_nodes_id.end()
				&& std::find(elem.m_nodes_id.begin(), elem.m_nodes_id.end(), a_id2) != elem.m_nodes_id.end();

			});

		if (currentIter != endIter)
		{
			result.push_back(*currentIter);
			currentIter++;
		}
	}

	return result;
}

//,      ID 
std::set<int> MeshLoader::getSurfaceNodesByID(int a_id)
{
	std::set<int> result;
	std::vector<BoundaryFiniteElement>::iterator currIter = m_BoundaryFiniteElements.begin();
	std::vector<BoundaryFiniteElement>::iterator endIter = m_BoundaryFiniteElements.end();

	while (currIter != endIter)
	{
		currIter = std::find_if(currIter, endIter, 
			[a_id](const BoundaryFiniteElement& element) 
			{
				return element.m_boundary_id == a_id;
			});

		if (currIter != endIter)
		{
			for (const auto& i : currIter->m_surface_elems_id)
			{
				result.insert(i);
			}
			++currIter;
		}
	}

	return result;
}

//,      ID 
std::vector<FiniteElement> MeshLoader::getFiniteElementsByID(int a_id)
{
	std::vector<FiniteElement> res;
	std::vector<FiniteElement>::iterator currIter = m_FiniteElements.begin();
	std::vector<FiniteElement>::iterator endIter = m_FiniteElements.end();

	while (currIter != endIter)
	{
		currIter = std::find_if(currIter, endIter, 
			[a_id](const FiniteElement& elem) {
				return elem.m_material_id == a_id;
			});

		if (currIter != endIter)
		{
			res.push_back(*currIter);
			currIter++;
		}
	}

	return res;
}

//,       ID 
std::vector<BoundaryFiniteElement> MeshLoader::getBoundaryFiniteElementsByID(int a_id)
{
	std::vector<BoundaryFiniteElement> result;
	std::vector<BoundaryFiniteElement>::iterator currIter = m_BoundaryFiniteElements.begin();
	std::vector<BoundaryFiniteElement>::iterator endIter = m_BoundaryFiniteElements.end();

	while (currIter != endIter)
	{
		currIter = std::find_if(currIter, endIter,
			[a_id](const BoundaryFiniteElement& element) {
				return element.m_boundary_id == a_id;
			});
		if (currIter != endIter)
		{
			result.push_back(*currIter);
			++currIter;
		}
	}

	return result;
}


//,         (  )
void MeshLoader::insertNewNode()
{
	std::unordered_set<Boundary, Hash> boundaries;

	for (auto& i : m_FiniteElements)
	{
		std::vector<int> currNodes(i.m_nodes_id);
		for (int j = 0; j < currNodes.size() - 1; ++j)
		{
			for (int k =  j + 1; k < currNodes.size(); ++k)
			{
				Boundary iBound(i.m_nodes_id[j], i.m_nodes_id[k], 0);
				std::unordered_set<Boundary, Hash>::iterator fBound = boundaries.find(iBound);

				if (fBound == boundaries.end())
				{
					Node lNode = m_Nodes.at(i.m_nodes_id[j]);
					Node rNode = m_Nodes.at(i.m_nodes_id[k]);
					int cNodeId = m_Nodes.end()->m_id;

                    //  -  ,   Node  
                    Node newCenterNode(cNodeId, (rNode.x + lNode.x) / 2,
												(rNode.y + lNode.y) / 2,
												(rNode.z + lNode.z) / 2, false);

					iBound.changeId(cNodeId);
					m_Nodes.push_back(newCenterNode);
					i.m_nodes_id.push_back(cNodeId);
					boundaries.insert(iBound);
				}
				else
				{
                    //  m_nodes[1]?
                    // ,     i.m_nodes_id,
                    //        j  k
                    //   
					i.m_nodes_id.push_back(fBound->m_nodes[1]);
				}
			}
		}
		i.m_nodes_id.shrink_to_fit();
	}

	for (auto& i : m_BoundaryFiniteElements)
	{
		for (int j = 0; j < i.m_surface_elems_id.size() - 1; ++j)
		{
			for (int k = j + 1; k < i.m_surface_elems_id.size(); ++k)
			{
				Boundary iBound(i.m_surface_elems_id[j], i.m_surface_elems_id[j], 0);
				std::unordered_set<Boundary, Hash>::iterator fBound = boundaries.find(iBound);
				if (fBound != boundaries.end())
				{
					i.m_surface_elems_id.push_back(fBound->m_nodes[1]);
				}
			}
		}
		i.m_surface_elems_id.shrink_to_fit();
	}
	m_BoundaryFiniteElements.shrink_to_fit();
	m_Nodes.shrink_to_fit();
}

//,  , n-           n
std::vector<std::set<int>> MeshLoader::getNeighborElements()
{
    //   , -   
	std::vector<std::set<int>> result(m_Nodes.size());
	for(int i = 0; i < m_Nodes.size(); i++)
	{
		std::set<int> neighbors;

        //         
        //   ,   i,    
		for (const auto& elem : m_FiniteElements)
		{
			for (const auto elem_id : elem.m_nodes_id)
			{
				if (elem_id != i)
				{
					neighbors.insert(elem_id);
				}
			}
		}

		result.push_back(neighbors);
	}

	return result;
}

