#include "Structures.h"

std::ostream& operator<<(std::ostream& out, const Node& a_node)
{
	out << a_node.m_id << "{" << a_node.x << "," <<
		a_node.y << "," << a_node.z << "}" << "(" << a_node.m_flag << ")" << std::endl;
	return out;
}


std::ostream& operator<<(std::ostream& out, const FiniteElement& a_element)
{
	out << a_element.m_id << " " << a_element.m_material_id << "(";
	for (int i : a_element.m_nodes_id)
	{
		out << i << "	";
	}
	out << ")";
	return out;
}


std::ostream& operator<<(std::ostream& out, const BoundaryFiniteElement& a_element)
{
	out << a_element.m_id << " " << a_element.m_boundary_id << " (";
	for (int i : a_element.m_surface_elems_id)
	{
		out << i << "	";
	}
	return out;
} 

Node::Node(int p_id, float p_x, float p_y, float p_z, bool p_flag)
	:m_id(p_id), m_flag(p_flag), x(p_x), y(p_y), z(p_z)
{};

void Boundary::changeId(int a_id)
{
	m_nodes[2] = a_id;
}

bool Boundary::operator==(const Boundary& a_node) const
{
	return a_node.m_nodes == m_nodes;
}

bool Boundary::operator<(const Boundary& a_node) const
{
	return m_nodes[2] < a_node.m_nodes[2];
}


void Hash::evaluate(size_t& a_seed, const int a_element) const
{
	a_seed ^= std::hash<int>()(a_element) + 0x9e3779b9 + (a_seed << 6) + (a_seed >> 2);
}

size_t Hash::operator()(const Boundary& p) const
{
	size_t seed = 0;
	
	int firstId = std::min(std::min(p.m_nodes[0], p.m_nodes[2]), p.m_nodes[1]);
	int lastId =  std::max(std::max(p.m_nodes[0], p.m_nodes[2]), p.m_nodes[1]);
	int MiddleId = (p.m_nodes[0] + p.m_nodes[2] + p.m_nodes[1]) - (firstId + lastId);

	evaluate(seed, firstId);
	evaluate(seed, lastId);
	evaluate(seed, MiddleId);

	return seed;
}

bool Node::operator==(const Node& a_node)
{
	return a_node.m_id == m_id &&
		   a_node.m_flag == m_flag &&
		   a_node.x == x &&
		   a_node.y == y &&
		   a_node.z == z;
}

bool Node::operator<(const Node& a_node)
{
	return m_id < a_node.m_id;
}
